import 'package:http/http.dart';
import 'package:rec_api_dart/src/api_paths.dart';
import 'package:rec_api_dart/src/entities/account.ent.dart';
import 'package:rec_api_dart/src/entities/account_permission.ent.dart';
import 'package:rec_api_dart/src/entities/dto/create_permission_data.dart';
import 'package:rec_api_dart/src/entities/dto/new_account_data.dart';
import 'package:rec_api_dart/src/entities/map/map_account_data.dart';
import 'package:rec_api_dart/src/entities/map/map_search_data.dart';
import 'package:rec_api_dart/src/env.base.dart';
import 'package:rec_api_dart/src/interceptors/user_token_interceptor.dart';
import 'package:rec_api_dart/src/interfaces/api_list_response.dart';
import 'package:rec_api_dart/src/services/base_service.dart';

/// This service is in charge of all requests related to [Account]
///
/// * **Authentication Required** - the user must be authenticated
class AccountsService extends ServiceBase {
  AccountsService({required EnvBase env, Client? client})
      : super(
          env: env,
          client: client,
          interceptors: [UserTokenInterceptor()],
        );

  /// Used to get an account by it's [id]
  Future<Account> getOne(String id) async {
    final path = ApiPaths.accounts.append(id).asUri(env.API_URL);

    return this.get(path).then(_mapToObject);
  }

  Future<ApiListResponse<MapAccountData>> search(MapSearchData searchData) async {
    final pathWithParams = ApiPaths.accountsSearch
        .withQueryParams(
          searchData.toJson(),
        )
        .asUri(env.API_URL);

    return this.get(pathWithParams).then(_mapToApiListReponse);
  }

  Future addUserToAccount(String? accountId, CreatePermissionData data) {
    final path = ApiPaths.accountsAddUser.append(accountId).asUri(env.API_URL);

    return this.post(path, {
      'user_dni': data.dni,
      'role': data.role,
    });
  }

  Future updateUserInAccount(String? accountId, int? permissionId, String role) {
    final path = ApiPaths.accountsEditRole.appendMultiple([
      accountId,
      permissionId.toString(),
    ]).asUri(env.API_URL);

    return this.post(path, {'role': role});
  }

  /// Delete a user's permission from this account
  Future deleteUserFromAccount(String? accountId, int? permissionId) {
    final path = ApiPaths.accountsAddUser
        .appendMultiple([accountId, permissionId.toString()]).asUri(env.API_URL);

    return this.delete(path);
  }

  /// Delete a [BankCard] from this account
  Future deleteBankCardAccount(String bankCardId) {
    final path = ApiPaths.removeCreditCard.append(bankCardId.toString()).asUri(env.API_URL);

    return this.delete(path);
  }

  /// Creates a new account linked to the authenticated user
  Future addAccount(NewAccountData data) {
    final path = ApiPaths.accountsAddNew.asUri(env.API_URL);

    return this.post(path, data.toJson());
  }

  Future updateAccount(String? accountId, Map<String, dynamic> data) {
    final path = ApiPaths.accounts.append(accountId).asUri(env.API_URL);

    return this.put(path, data);
  }

  Future<ApiListResponse<AccountPermission>> listAccountPermissions(
    String? accountId,
  ) {
    final path = ApiPaths.accountsPermissions.append(accountId).asUri(env.API_URL);

    return this.get(path).then(_mapPermissions);
  }

  // TODO: Look for a way of improving how we build ApiListResponse
  // It works as is, but there's a lot of repetition that would be nice to remove
  // if we find a good way of doing it
  ApiListResponse<MapAccountData> _mapToApiListReponse(Map<String, dynamic> data) {
    return ApiListResponse<MapAccountData>.fromJson(
      data['data'],
      mapper: (el) => MapAccountData.fromJson(el),
    );
  }

  ApiListResponse<AccountPermission> _mapPermissions(Map<String, dynamic> data) {
    return ApiListResponse<AccountPermission>.fromJson(
      data['data'],
      mapper: (el) => AccountPermission.fromJson(el),
    );
  }

  Account _mapToObject(Map<String, dynamic> data) => Account.fromJson(data['data']);
}
